package com.simuwang.base.mapper;

import com.simuwang.base.pojo.dos.EmailTemplateApplicationRuleDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

@Mapper
public interface EmailTemplateApplicationRuleMapper {

    List<EmailTemplateApplicationRuleDO> queryByTemplateId(@Param("templateIdList") List<Integer> templateIdList);

    void batchInsert(@Param("itemDoList") List<EmailTemplateApplicationRuleDO> templateApplicationRuleDOList);

    void batchUpdate(@Param("itemDoList") List<EmailTemplateApplicationRuleDO> editList);

    void deleteByTemplateId(@Param("templateIdList")List<Integer> idList,@Param("userId") Integer userId);

    void deleteByIdList(@Param("idList") List<Integer> idList,@Param("userId") Integer userId);
}
